﻿using Conductor.Client;
using Conductor.Domain.Models;
using Hotline.CallCenter.BlackLists;
using Hotline.CallCenter.Devices;
using Hotline.CallCenter.Ivrs;
using Hotline.Realtimes;
using Hotline.Repository.SqlSugar;
using Hotline.Settings;
using Hotline.Share.Dtos.Realtime;
using Hotline.Users;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using NewRock.Sdk;
using NewRock.Sdk.Security;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Cache;
using XF.Domain.Entities;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class TestController : BaseController
    {
        private readonly ILogger<TestController> _logger;
        private readonly IAuthorizeGenerator _authorizeGenerator;
        private readonly IOptionsSnapshot<DeviceConfigs> _options;
        private readonly ISessionContext _sessionContext;
        private readonly IUserRepository _userRepository;

        private readonly ITypedCache<User> _cache;
        private readonly IRealtimeService _realtimeService;
        private readonly IBlacklistDomainService _blacklistDomainService;
        private readonly IIvrDomainService _ivrDomainService;
        private readonly ISugarUnitOfWork<HotlineDbContext> _uow;
        private readonly IWorkflowRemotingService _workflowRemotingService;

        //private readonly ITypedCache<List<User>> _cache;
        //private readonly ICacheManager<User> _cache;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="client"></param>
        /// <param name="logger"></param>
        /// <param name="authorizeGenerator"></param>
        /// <param name="options"></param>
        /// <param name="sessionContext"></param>
        /// <param name="userRepository"></param>
        /// <param name="cache"></param>
        /// <param name="realtimeService"></param>
        /// <param name="blacklistDomainService"></param>
        /// <param name="ivrDomainService"></param>
        public TestController(
            INewRockClient client,
            ILogger<TestController> logger,
            IAuthorizeGenerator authorizeGenerator,
            IOptionsSnapshot<DeviceConfigs> options,
            ISessionContext sessionContext,
            IUserRepository userRepository,
            //ICacheManager<User> cache
            //ITypedCache<List<User>> cache
            ITypedCache<User> cache,
            IRealtimeService realtimeService,
            IBlacklistDomainService blacklistDomainService,
            IIvrDomainService ivrDomainService,
            ISugarUnitOfWork<HotlineDbContext> uow,
            IWorkflowRemotingService workflowRemotingService
            )
        {
            _logger = logger;
            _authorizeGenerator = authorizeGenerator;
            _options = options;
            _sessionContext = sessionContext;
            _userRepository = userRepository;
            _cache = cache;
            _realtimeService = realtimeService;
            _blacklistDomainService = blacklistDomainService;
            _ivrDomainService = ivrDomainService;
            _uow = uow;
            _workflowRemotingService = workflowRemotingService;
        }


        /// <summary>
        /// signalR测试（method: Ring）
        /// </summary>
        /// <returns></returns>
        [HttpGet("ring")]
        public async Task RingTest()
        {
            await _realtimeService.RingAsync(_sessionContext.RequiredUserId, new RingDto { From = _sessionContext.Phone ?? "未知号码" }, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("t2")]
        public async Task GetVoiceEndAnswerAsyncTest()
        {
            //var answer = await _ivrDomainService.GetVoiceEndAnswerAsync("3", HttpContext.RequestAborted);
            //Console.WriteLine(answer.Content);
            throw new UserFriendlyException(2001, "test");
        }

        [AllowAnonymous]
        [HttpGet("wfdefine")]
        public async Task<IReadOnlyList<Definition>> GetWorkflowDefine()
        {
            var a = await _workflowRemotingService.QueryDefinitionAsync(HttpContext.RequestAborted);
            return a.Result;
        }

        [ApiExplorerSettings(IgnoreApi = true)]
        [AllowAnonymous]
        [HttpGet("cdb")]
        public Task CreateTableMultiple()
        {
            var db = _uow.Db;
            db.DbMaintenance.CreateDatabase();

            var types = typeof(User).Assembly.GetTypes()
                .Where(d => d.GetInterfaces().Any(x => x == typeof(IEntity)))
                .ToArray();

            db.CodeFirst.InitTables(types);//根据types创建表

            return Task.CompletedTask;
        }
    }
}
