﻿using Hotline.Settings;
using Hotline.Share.Enums;
using Microsoft.Extensions.DependencyInjection;
using Org.BouncyCastle.Asn1.X509;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar.DataPermissions;

public class DataPermissionManager : IDataPermissionManager, IScopeDependency
{
    private readonly IServiceScopeFactory _serviceScopeFactory;

    public DataPermissionManager(IServiceScopeFactory serviceScopeFactory)
    {
        _serviceScopeFactory = serviceScopeFactory;
    }

    public DataPermissionScheme GetQueryFilter<TEntity>(ISessionContext sessionContext) where TEntity : class, IEntity<string>, IDataPermission, new()
    {
        EAuthorityType? queryFilter = GetCurrentQueryFilter(sessionContext.Roles, typeof(TEntity).Name);
        if (queryFilter == null)
        {
            
            //todo 未配置数据查询权限处理方案
            //TODO 不处理
        }

        //全部
        if (queryFilter == EAuthorityType.All)
            return new DataPermissionScheme(EAuthorityType.All);

        //本部
        if (queryFilter == EAuthorityType.Org)
            return new DataPermissionScheme(EAuthorityType.Org);

        //本部及本部以下
        if (queryFilter == EAuthorityType.OrgAndBelow)
        {
            //todo 查询对应部门编码
            return new DataPermissionScheme(EAuthorityType.OrgAndBelow, "");
        }

        //创建人
        if (queryFilter == EAuthorityType.Create)
            return new DataPermissionScheme(EAuthorityType.Create);

        throw new NotImplementedException();
    }

    private EAuthorityType? GetCurrentQueryFilter(string[] roles, string entityName)
    {
        using var scope = _serviceScopeFactory.CreateScope();
        var systemDataTableRepository = scope.ServiceProvider.GetService<ISystemDataTableRepository>();
        var systemDataAuthorityRepository = scope.ServiceProvider.GetService<ISystemDataAuthorityRepository>();

        ////查询对应表配置
        var tableModel = systemDataTableRepository!.GetAsync(x => x.EntityName == entityName).Result;
        ////查询表对应最高数据权限
        var auth = systemDataAuthorityRepository!.GetMyTopDataAuth(roles, tableModel!.Id);
        return auth;
    }
}